%======================================================================================
% Example of "sporting club" LS: interpolation of a circle
% Eq. of the circle:
%  (x - x0)^2 + (y - y0)^2 = R^2
%  x^2 - 2*x*x0 + x0^2 + y^2 - 2*y*y0 + y0^2 = R^2
% Model:
%  x^2 + y^2 ~= [2x  2y  1] [x0  y0  a]',    where a = R^2 - x_0^2 - y_0^2 
%
% WARNING: the regressor IS correlated with the noise, so the estimate is
% doomed to be biased. Anyway, let's give it a try...
%======================================================================================

%--------------------------------------------------------------------------------------
% A. True parameters
%--------------------------------------------------------------------------------------
true_x0 = 2;
true_y0 = 3;
true_R = 1.5;

%--------------------------------------------------------------------------------------
% B. Plot the true circle and some random points (random angles, + bivariate Gaussian random noise)
%--------------------------------------------------------------------------------------
NPLOT = 1000;
plotangles = linspace(-pi, pi, NPLOT)';
xplot_true = true_R*cos(plotangles) + true_x0;
yplot_true = true_R*sin(plotangles) + true_y0;

% random points
N = 120;
noise_stddev = 0.2;
randangles = 2*pi*rand(N,1);
X =  true_R*cos(randangles) + true_x0 + noise_stddev*randn(N,1);
Y =  true_R*sin(randangles) + true_y0 + noise_stddev*randn(N,1);

figure(1);
hold on;
axis([0 5 0 5])
plot(xplot_true, yplot_true, 'b');
plot(X, Y, 'k.');

%--------------------------------------------------------------------------------------
% C. Estimate the circle with least squares method and plot it in red
%--------------------------------------------------------------------------------------

Phi = [2*X, 2*Y, ones(N,1)];
V = [X.^2 + Y.^2];   % the variable that we regress, i.e. the one that usually we call Y
thetaLS = inv(Phi'*Phi)*Phi'*V;

x0 = thetaLS(1);
y0 = thetaLS(2);
R = sqrt(thetaLS(3) + x0^2 + y0^2);

xplot = R*cos(plotangles) + x0;
yplot = R*sin(plotangles) + y0;

plot(xplot, yplot, 'r');

pause;


