%======================================================================================
% CONSTRAINED LEAST SQUARES
%
% Example: fitting with a conditional model:
%   y = a + bx            if x <= x0
%   y = c + dx + ex^2     if x >  x0
%
% (The example is taken from A. Azzalini, Inferenza statistica [...], Springer)
%======================================================================================

%--------------------------------------------------------------------------------------
% A. True parameters
%--------------------------------------------------------------------------------------
x0 = 2;
true_a = 1;
true_b = -1;
true_e = 0.3;

true_d = true_b - 2*true_e*x0;
true_c = true_a + true_b*x0 - true_d*x0 - true_e*x0^2;

true_theta = [true_a; true_b; true_c; true_d; true_e];

%--------------------------------------------------------------------------------------
% B. Plot the true function y = f(x, true_theta) and some noisy measures: y_i = f(x_i, true_theta) + noise_i
%--------------------------------------------------------------------------------------
NPLOT = 1000;
xplot = linspace(0, 5, NPLOT)';

Phiplot = [ones(NPLOT,1).*(xplot <= x0), xplot.*(xplot <= x0), ones(NPLOT,1).*(xplot > x0), xplot.*(xplot > x0), xplot.^2.*(xplot > x0)];
yplot_true = Phiplot*true_theta;

N = 120;
X = 5*rand(N, 1);
Phi = [ones(N,1).*(X <= x0), X.*(X <= x0), ones(N,1).*(X > x0), X.*(X > x0), X.^2.*(X > x0)];
Y = Phi*true_theta + 0.5*randn(N,1);

figure(1);
hold on;
axis([0 5 -3 3])
plot([x0 x0], [-3 3], 'k');
plot(xplot, yplot_true, 'k');
plot(X, Y, 'k.');

pause;

%--------------------------------------------------------------------------------------
% C. Estimate the parameter with least squares method
% NAIVE: UNCONSTRAINED LS
%--------------------------------------------------------------------------------------

P = inv(Phi'*Phi);
thetaLS = P*Phi'*Y;   % inv(Phi'*Phi)*Phi'*Y;

%figure(2);
%hold on;

plot(X, Y, 'k.');
ymodel = Phiplot*thetaLS;
plot(xplot, ymodel, 'r');

pause;

%--------------------------------------------------------------------------------------
% D. Estimate the parameter with least squares method
% CONTINUOUS AND DIFFERENTIABLE: CONSTRAINED LS
%--------------------------------------------------------------------------------------

K = [1, x0, -1, -x0, -x0^2;
     0,  1,  0,  -1, -2*x0 ];

thetaCLS = P*Phi'*Y - P*K'*inv(K*P*K')*K*P*Phi'*Y ;
ymodelc = Phiplot*thetaCLS;
plot(xplot, ymodelc, 'b');
plot(xplot, yplot_true, 'k');

pause;


